package Pooru::API::V0::Controller::Tags;
use Mojo::Base "Mojolicious::Controller", -signatures;

my %search_opts = Pooru::API::V0::_search_opts->%*;

sub list ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(
		json => {error => "Invalid page number."},
		status => 400,
	) if $v->has_error;

	my $paged_tags = $self->schema->resultset("TagCountView")
	    ->search(undef, $search_opts{TagCountView})->page($page);
	my @tags = map +{
		id => $_->id,
		name => $_->name,
		kind_id => $_->kind_id,
		display => $_->display,
		count => $_->count,
	}, $paged_tags->all;

	return $self->render(json => {
		tags => [@tags],
		pager => {$self->pager($paged_tags)},
	});
}

sub show ($self)
{
	my $v = $self->validation;

	my $tag_id_or_name = $v->optional("id")->num(1, undef)->param //
	    $v->optional("display")->param //
	    $v->required("name")->param;
	return $self->render(
		json => {error => "Invalid tag ID or name."},
		status => 400,
	) if $v->has_error;

	my %search = ($v->topic => $tag_id_or_name);

	my $kind_id = $v->optional("kind_id")->param;
	return $self->render(
		json => {error => "Invalid kind ID."},
		status => 400,
	) if $v->has_error;
	$search{kind_id} = $kind_id if defined($kind_id);

	my @tags = map +{
		id => $_->id,
		name => $_->name,
		kind_id => $_->kind_id,
		display => $_->display,
		count => $_->count,
	}, $self->schema->resultset("TagCountView")->search(\%search)->all;

	return $self->render(
		json => {error => "Tag not found."},
		status => 404,
	) if @tags == 0;

	return $self->render(json => {tags => [@tags]});
}

sub search ($self)
{
	my $v = $self->validation;

	my $q = $v->required("q")->param;
	return $self->render(
		json => {error => "Missing query."},
		status => 400,
	) if $v->has_error;

	return $self->render(
		json => {tags => []},
		status => 200
	) if $q eq "";

	$q =~ s/[%\\_]/\\$&/g;
	$q .= '%';
	my %attrs = (
		$search_opts{TagCountView}->%*,
		rows => 10,
	);

	my @tags = map +{
		id => $_->id,
		name => $_->name,
		kind_id => $_->kind_id,
		display => $_->display,
		count => $_->count,
	}, $self->schema->resultset("TagCountView")
	    ->search({name => \["LIKE ? ESCAPE '\\'", $q]}, \%attrs);

	return $self->render(json => {tags => [@tags]});
}

1;
